/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Author
    Hrvoje Jasak, Wikki Ltd.  All rights reserved

\*---------------------------------------------------------------------------*/

#include "fieldStorage.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class Type>
fieldStorage<Type>::fieldStorage()
:
    storedField_(),
    curTimeIndex_(-1)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //


template<class Type>
void fieldStorage<Type>::store()
{
    if (curTimeIndex_ != time_.timeIndex())
    {
        storedField_ = *this;
        curTimeIndex_ = time_.timeIndex();
    }
}


template<class Type>
const Field<Type>& originalPatchField<Type>::store() const
{
    if (curTimeIndex_ == time_.timeIndex())
    {
        return storedField_;
    }
    else
    {
        return *this;
    }
};


template<class Type>
void originalPatchField<Type>::autoMap
(
    const fvPatchFieldMapper& m
)
{
    storedField_::autoMap(m);
}


template<class Type>
void clear() const
{
    originalPatchField_.setSize(0);
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
