/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Class
    adiabaticFvPatchScalarField

Description
    adiabatic boundary (with radiation)

    Example usage:
        myWallPatch
        {
            type            adiabatic;
            K               K;                 // Name of K field
            value           300.0;             // Initial temperature / [K]
        }


SourceFiles
    adiabaticFvPatchScalarField.C

\*---------------------------------------------------------------------------*/

#ifndef adiabaticFvPatchScalarField_H
#define adiabaticFvPatchScalarField_H

#include "mixedFvPatchFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
      Class adiabaticFvPatchScalarField Declaration
\*---------------------------------------------------------------------------*/

class adiabaticFvPatchScalarField
:
    public mixedFvPatchScalarField
{
    // Private data

        //- Name of thermal conductivity field
        word KName_;


public:

    //- Runtime type information
    TypeName("adiabatic");


    // Constructors

        //- Construct from patch and internal field
        adiabaticFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        adiabaticFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given
        // adiabaticFvPatchScalarField
        // onto a new patch
        adiabaticFvPatchScalarField
        (
            const adiabaticFvPatchScalarField&,
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        adiabaticFvPatchScalarField
        (
            const adiabaticFvPatchScalarField&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchScalarField> clone() const
        {
            return tmp<fvPatchScalarField>
            (
                new adiabaticFvPatchScalarField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        adiabaticFvPatchScalarField
        (
            const adiabaticFvPatchScalarField&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchScalarField> clone
        (
            const DimensionedField<scalar, volMesh>& iF
        ) const
        {
            return tmp<fvPatchScalarField>
            (
                new adiabaticFvPatchScalarField(*this, iF)
            );
        }


    // Member functions

        // Evaluation functions

            //- Update the coefficients associated with the patch field
            virtual void updateCoeffs();


        // Mapping functions


        // I-O

            //- Write
            void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
