/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Class
    POD

Author
    Hrvoje Jasak, Wikki Ltd.  All rights reserved.

Description
    Proper Orthogonal Decomposition of fields.

SourceFiles
    PODI.H
    POD.C
    PODIO.C

\*---------------------------------------------------------------------------*/

#ifndef POD_H
#define POD_H

#include "volFieldsFwd.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                     Namespace POD functions Declaration
\*---------------------------------------------------------------------------*/

namespace POD
{
    //- Dot product of snapshots
    template<class Type>
    Type projection
    (
        const GeometricField<Type, fvPatchField, volMesh>& a,
        const GeometricField<Type, fvPatchField, volMesh>& b
    );

    template<class Type>
    Type projection
    (
        const tmp<GeometricField<Type, fvPatchField, volMesh> >& ta,
        const GeometricField<Type, fvPatchField, volMesh>& b
    );

}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "POD.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
