/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "surfaceInterpolationScheme.H"
#include "VectorNFieldTypes.H"
#include "linear.H"
#include "reverseLinear.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#define makeBaseSurfaceInterpolationScheme(Type)                              \
                                                                              \
defineNamedTemplateTypeNameAndDebug(surfaceInterpolationScheme<Type>, 0);     \
                                                                              \
defineTemplateRunTimeSelectionTable                                           \
(                                                                             \
    surfaceInterpolationScheme<Type>,                                         \
    Mesh                                                                      \
);                                                                            \
                                                                              \
defineTemplateRunTimeSelectionTable                                           \
(                                                                             \
    surfaceInterpolationScheme<Type>,                                         \
    MeshFlux                                                                  \
);


#define doMakeSchemes(type, Type, args...)                                    \
    makeBaseSurfaceInterpolationScheme(type)                                  \
                                                                              \
    makeSurfaceInterpolationTypeScheme(linear, type)                          \
    makeSurfaceInterpolationTypeScheme(reverseLinear, type)

forAllVectorNTypes(doMakeSchemes)

forAllTensorNTypes(doMakeSchemes)

forAllDiagTensorNTypes(doMakeSchemes)

forAllSphericalTensorNTypes(doMakeSchemes)

#undef doMakeSchemes

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
