/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Namespace
    DenseMatrixTools

SourceFiles
    DenseMatrixTools.C

\*---------------------------------------------------------------------------*/

#ifndef DenseMatrixTools_H
#define DenseMatrixTools_H

#include "SquareMatrix.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                      Class DenseMatrixTools Declaration
\*---------------------------------------------------------------------------*/

namespace DenseMatrixTools
{
    //- Solve using Gauss Elimination with pivoting.  Destroys matrix and b
    template<class Type>
    static void solve(SquareMatrix<Type>& A, List<Type>& x, List<Type>& b);

    //- Q-R decomposition
    template<class Form, class Type>
    static void qrDecompose
    (
        const label nCols,
        FieldField<Field, Type>& A,
        Matrix<Form, Type>& R
    );
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "DenseMatrixTools.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
