/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Description
    Dimensioned diagonal tensor obtained from generic dimensioned type.

\*---------------------------------------------------------------------------*/

#include "dimensionedDiagTensor.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //


// * * * * * * * * * * * * * * * Friend Functions  * * * * * * * * * * * * * //

dimensionedScalar tr(const dimensionedDiagTensor& ddt)
{
    return dimensionedScalar
    (
        "tr(" + ddt.name() + ')',
        ddt.dimensions(),
        tr(ddt.value())
    );
}


dimensionedScalar det(const dimensionedDiagTensor& ddt)
{
    return dimensionedScalar
    (
        "det(" + ddt.name() + ')',
        pow(ddt.dimensions(), tensor::dim),
        det(ddt.value())
    );
}


dimensionedDiagTensor inv(const dimensionedDiagTensor& ddt)
{
    return dimensionedDiagTensor
    (
        "inv(" + ddt.name() + ')',
        dimless/ddt.dimensions(),
        inv(ddt.value())
    );
}


dimensionedDiagTensor diag(const dimensionedTensor& dt)
{
    return dimensionedDiagTensor
    (
        "diag(" + dt.name() + ')',
        dt.dimensions(),
        diag(dt.value())
    );
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
