/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Class
    BlockGaussSeidelPrecon

Description
    Template specialisation for tensor block Gauss-Seidel preconditioning

SourceFiles
    tensorBlockGaussSeidelPrecon.C

\*---------------------------------------------------------------------------*/

#ifndef tensorBlockGaussSeidelPrecon_H
#define tensorBlockGaussSeidelPrecon_H

#include "BlockGaussSeidelPrecon.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

template<>
template<>
void BlockGaussSeidelPrecon<tensor>::BlockSweep
(
    Field<tensor>& x,
    const Field<tensor>& dD,
    const Field<tensor>& upper,
    const Field<tensor>& b
) const;


template<>
template<>
void BlockGaussSeidelPrecon<tensor>::BlockSweep
(
    Field<tensor>& x,
    const Field<tensor>& dD,
    const Field<tensor>& upper,
    const Field<tensor>& lower,
    const Field<tensor>& b
) const;


template<>
void BlockGaussSeidelPrecon<tensor>::precondition
(
    tensorField& x,
    const tensorField& b
) const;


template<>
void BlockGaussSeidelPrecon<tensor>::preconditionT
(
    tensorField& xT,
    const tensorField& bT
) const;


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
