/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Class
    solidInterface

\*---------------------------------------------------------------------------*/

#include "solidInterface.H"
#include "volFields.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

autoPtr<solidInterface> solidInterface::New
(
 const word& name,
 const fvMesh& mesh,
 const constitutiveModel& rheology
 )
{
    Info<< "Selecting solidInterface procedure " << name << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_->find(name);

    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalIOErrorIn
            (
                "solidInterface::New(\n"
                "    const word& name,\n"
                "    const fvMesh& mesh,\n"
                "    const constitutiveModel& rheology" //,\n"
                ")",
                rheology
                )   << "Unknown solidInterfaceMethod "
                    << name << endl << endl
                    << "Valid  solidInterfaces are : " << endl
                    << dictionaryConstructorTablePtr_->sortedToc()
                    << exit(FatalIOError);
    }

    return autoPtr<solidInterface>(cstrIter()(name, mesh, rheology));
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
